package Mentat::Processor::Engine::Convertor;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Convertor -

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Convertor;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use DB_File;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Mentat::Message::Convertor::IDEA;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA %STATE_STORAGE);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     : Emit/execute event 'message_hello' with appropriate arguments
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($messages, @args) = @_;
    confess "Messages must be given as ARRAY reference" unless $messages and ref $messages eq 'ARRAY';

    Log::Loger->debug(__PACKAGE__, "[ENG:Convertor] Converting ".(scalar @$messages)." messages");

    # Convert the messages only if the convertor was set
    if ($self->{CONVERTOR}) {
        my $tmp = [];
        my $cnt = 0;
        foreach my $msg (@$messages) {
            eval {
                push(@$tmp, $self->{CONVERTOR}->convert($msg));
            };
            if ($@) {
                Log::Loger->error(__PACKAGE__, "[ENG:Convertor] Conversion error (skipping message): $@");
            }
        }
        $messages = $tmp;
    }

    Log::Loger->notice(__PACKAGE__, "[ENG:Convertor] Conversion complete");
    return (EC_CONTINUE, $messages, @args);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($$;$) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    $self->{CONVERTOR} = Mentat::Message::Convertor::IDEA->new(%config);

    return $self;
}

=pod

=back

=cut

1;
